<?php
namespace FiloBlu\Storelocator\Model\Import;

use FiloBlu\Storelocator\Model\Import\Stores\RowValidatorInterface as ValidatorInterface;
use Magento\ImportExport\Model\Import\ErrorProcessing\ProcessingErrorAggregatorInterface;
use Magento\Framework\App\ResourceConnection;

class Stores extends \Magento\ImportExport\Model\Import\Entity\AbstractEntity
{

    const ID = 'store_id';
    const ENABLED = 'enabled';
    const TITLE = 'title';
    const LATITUDE = 'latitude';
    const LONGITUDE = 'longitude';
    const STORE_IDS = 'store_ids';
    const STREET = 'street';
    const POSTCODE = 'postcode';
    const CITY = 'city';
    const COUNTRY = 'country';
    const PHONE_1 = 'phone_1';
    const PHONE_2 = 'phone_2';
    const FAX = 'fax';
    const EMAIL = 'email';
    const WEBSITE_URL = 'website_url';
    const ADDITIONAL_TEXT = 'additional_text';

    const TABLE_Entity = 'filoblu_storelocator_stores';

    /**
     * Validation failure message template definitions
     *
     * @var array
     */
    protected $_messageTemplates = [
        ValidatorInterface::ERROR_TITLE_IS_EMPTY => 'TITLE is empty',
    ];

    protected $_permanentAttributes = [
        self::ENABLED,
        self::TITLE,
        self::STORE_IDS,
    ];

    /**
     * If we should check column names
     *
     * @var bool
     */
    protected $needColumnCheck = true;
    protected $storesFactory;
    /**
     * Valid column names
     *
     * @array
     */
    protected $validColumnNames = [

        self::ENABLED,
        self::TITLE,
        self::LATITUDE,
        self::LONGITUDE,
        self::STORE_IDS,
        self::STREET,
        self::POSTCODE,
        self::CITY,
        self::COUNTRY,
        self::PHONE_1,
        self::PHONE_2,
        self::FAX,
        self::EMAIL,
        self::WEBSITE_URL,
        self::ADDITIONAL_TEXT,
    ];

    /**
     * Need to log in import history
     *
     * @var bool
     */
    protected $logInHistory = true;

    protected $_validators = [];


    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    protected $_connection;
    protected $_resource;
    protected $_storeLocatorHelper;

    /**
     * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
     */
    public function __construct(
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Magento\ImportExport\Helper\Data $importExportData,
        \Magento\ImportExport\Model\ResourceModel\Import\Data $importData,
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\ImportExport\Model\ResourceModel\Helper $resourceHelper,
        \Magento\Framework\Stdlib\StringUtils $string,
        ProcessingErrorAggregatorInterface $errorAggregator,
        \FiloBlu\Storelocator\Model\Stores $storesFactory,
        \FiloBlu\Storelocator\Helper\Data $storeLocatorHelper
    )
    {
        $this->jsonHelper = $jsonHelper;
        $this->_importExportData = $importExportData;
        $this->_resourceHelper = $resourceHelper;
        $this->_dataSourceModel = $importData;
        $this->_resource = $resource;
        $this->_connection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $this->errorAggregator = $errorAggregator;
        $this->storesFactory = $storesFactory;
        $this->_storeLocatorHelper = $storeLocatorHelper;
    }

    public function getValidColumnNames()
    {
        return $this->validColumnNames;
    }

    /**
     * Entity type code getter.
     *
     * @return string
     */
    public function getEntityTypeCode()
    {
        return 'filoblustorelocator_import';
    }

    /**
     * Row validation.
     *
     * @param array $rowData
     * @param int $rowNum
     * @return bool
     */
    public function validateRow(array $rowData, $rowNum)
    {
        return true;
    }


    /**
     * Create Advanced price data from raw data.
     *
     * @throws \Exception
     * @return bool Result of operation.
     */
    protected function _importData()
    {
        if (\Magento\ImportExport\Model\Import::BEHAVIOR_APPEND == $this->getBehavior()) {
            $this->saveEntity();
        }

        return true;
    }

    public function saveEntity()
    {
        $behavior = $this->getBehavior();
        $i = 0;

        while ($bunch = $this->_dataSourceModel->getNextBunch()) {

            $entityList = [];
            foreach ($bunch as $rowNum => $rowData) {
                $i++;

                if (!$this->validateRow($rowData, $rowNum)) {
//                    $this->addRowError(ValidatorInterface::ERROR_TITLE_IS_EMPTY, $rowNum);
                    continue;
                }
                if ($this->getErrorAggregator()->hasToBeTerminated()) {
                    $this->getErrorAggregator()->addRowToSkip($rowNum);
                    continue;
                }

                if (!$rowData[self::LATITUDE] || $rowData[self::LONGITUDE]) {
                    $address = $rowData[self::STREET] . " " .
                        $rowData[self::POSTCODE] . " " .
                        $rowData[self::CITY] . " " .
                        $rowData[self::COUNTRY];
                    $geocodeData = $this->_storeLocatorHelper->geocode($address);
                    if ($geocodeData) {
                        $rowData[self::LATITUDE] = $geocodeData[0];
                        $rowData[self::LONGITUDE] = $geocodeData[1];
                    }
                }

                if (!$rowData[self::ENABLED]) {
                    $rowData[self::ENABLED] = 1;
                }

                if (!$rowData[self::STORE_IDS]) {
                    $rowData[self::STORE_IDS] = 0;
                }

                $_countryCode = $this->getCountryCodeByString($rowData[self::COUNTRY]);

                $entityList[$i][] = [

                    self::ENABLED => $rowData[self::ENABLED],
                    self::TITLE => $rowData[self::TITLE],
                    self::LATITUDE => $rowData[self::LATITUDE],
                    self::LONGITUDE => $rowData[self::LONGITUDE],
                    self::STORE_IDS => $rowData[self::STORE_IDS],
                    self::STREET => $rowData[self::STREET],
                    self::POSTCODE => $rowData[self::POSTCODE],
                    self::CITY => $rowData[self::CITY],
                    self::COUNTRY => $_countryCode,
                    self::PHONE_1 => $rowData[self::PHONE_1],
                    self::PHONE_2 => $rowData[self::PHONE_2],
                    self::FAX => $rowData[self::FAX],
                    self::EMAIL => $rowData[self::EMAIL],
                    self::WEBSITE_URL => $rowData[self::WEBSITE_URL],
                    self::ADDITIONAL_TEXT => $rowData[self::ADDITIONAL_TEXT],
                ];
            }

            $this->countItemsCreated += $i;

            if (\Magento\ImportExport\Model\Import::BEHAVIOR_APPEND == $behavior) {
                $this->saveEntityFinish($entityList, self::TABLE_Entity);
            }
        }

        return $this;
    }

    protected function saveEntityFinish(array $entityData, $table)
    {
        if ($entityData) {
            $tableName = $this->_connection->getTableName($table);
            $entityIn = [];
            foreach ($entityData as $id => $entityRows) {
                foreach ($entityRows as $row) {
                    $entityIn[] = $row;
                }
            }
            if ($entityIn) {
                $this->_connection->insertOnDuplicate($tableName, $entityIn, [
                    self::ENABLED,
                    self::TITLE,
                    self::LATITUDE,
                    self::LONGITUDE,
                    self::STORE_IDS,
                    self::STREET,
                    self::POSTCODE,
                    self::CITY,
                    self::COUNTRY,
                    self::PHONE_1,
                    self::PHONE_2,
                    self::FAX,
                    self::EMAIL,
                    self::WEBSITE_URL,
                    self::ADDITIONAL_TEXT,
                ]);
            }
        }
        return $this;
    }

    protected function getCountryCodeByString($_countryStr)
    {
        $country = "";

        switch (strtolower($_countryStr)) {
            case 'austria':
                $country = "AT";
                break;
            case 'belgium':
                $country = "BE";
                break;
            case 'czech republic':
                $country = "CZ";
                break;
            case 'france':
                $country = "FR";
                break;
            case 'germany':
                $country = "DE";
                break;
            case 'hungary':
                $country = "HU";
                break;
            case 'italy':
                $country = "IT";
                break;
            case 'san marino':
                $country = "SM";
                break;
            case 'luxembourg':
                $country = "LU";
                break;
            case 'netherlands':
                $country = "NL";
                break;
            case 'poland':
                $country = "PL";
                break;
            case 'slovakia':
                $country = "SK";
                break;
            case 'united kingdom':
                $country = "GB";
                break;
        }

        return $country;
    }


}